/*
    callback.h -- C++ callback mechanism
    Copyright (C) 2003-2018 Marc Lehmann <pcg@goof.com>
 
    Deliantra is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with gvpe; if not, write to the Free Software
    Foundation, Inc. 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef CALLBACK_H__
#define CALLBACK_H__

template<typename signature>
class callback;

template<typename R, typename ...Args>
class callback<R (Args...)>
{
  struct object { };

  typedef R (object::*ptr_type)(Args...);

  void *obj;
  R (object::*meth)(Args...);

  /* a proxy is a kind of recipe on how to call a specific class method	*/
  struct proxy_base {
    virtual R call (void *obj, R (object::*meth)(Args...), Args... args) const = 0;
  };
  template<class O1, class O2>
  struct proxy : proxy_base {
    virtual R call (void *obj, R (object::*meth)(Args...), Args... args) const
      {
        return (R)((reinterpret_cast<O1 *>(obj)) ->* (reinterpret_cast<R (O2::*)(Args...)>(meth)))
          (args...);
      }
  };

  proxy_base *prxy;

public:
  template<class O1, class O2>
  explicit callback (O1 *object, R (O2::*method)(Args...))
    {
      static proxy<O1,O2> p;
      obj  = reinterpret_cast<void *>(object);
      meth = reinterpret_cast<R (object::*)(Args...)>(method);
      prxy = &p;
    }

  R call(Args... args) const
    {
      return prxy->call (obj, meth, args...);
    }

  R operator ()(Args... args) const
    {
      return call (args...);
    }
};

#endif
