/*
 * This file is part of Deliantra, the Roguelike Realtime MMORPG.
 *
 * Copyright (©) 2017,2018 Marc Alexander Lehmann / the Deliantra team
 * Copyright (©) 2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016 Marc Alexander Lehmann / Robin Redeker / the Deliantra team
 * Copyright (©) 2002 Mark Wedel & Crossfire Development Team
 * Copyright (©) 1992 Frank Tore Johansen
 *
 * Deliantra is free software: you can redistribute it and/or modify it under
 * the terms of the Affero GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the Affero GNU General Public License
 * and the GNU General Public License along with this program. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * The authors can be reached via e-mail to <support@deliantra.net>
 */

#include <global.h>
#include <rmg.h>
#include <rproto.h>

/*  make a map and maze the floor.  */
void
maptile::make_map_floor (char **maze, const char *floorstyle, random_map_params *RP)
{
  clear ();

  width  = RP->Xsize;
  height = RP->Ysize;

  alloc ();

  /* get the style map */
  maptile *style_map = find_style ("/styles/floorstyles", floorstyle, RP->difficulty);
  if (!style_map)
    return;

  /* fill up the map with the given floor style */
  if (object *the_floor = style_map->pick_random_object (rmg_rndm))
    for (int x = 0; x < width; x++)
      for (int y = 0; y < height; y++)
        insert (the_floor->clone (), x, y, 0, INS_NO_MERGE | INS_NO_WALK_ON);
}

