/*
 * This file is part of Deliantra, the Roguelike Realtime MMORPG.
 *
 * Copyright (©) 2017,2018 Marc Alexander Lehmann / the Deliantra team
 * Copyright (©) 2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016 Marc Alexander Lehmann / Robin Redeker / the Deliantra team
 * Copyright (©) 2002 Mark Wedel & Crossfire Development Team
 * Copyright (©) 1992 Frank Tore Johansen
 *
 * Deliantra is free software: you can redistribute it and/or modify it under
 * the terms of the Affero GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the Affero GNU General Public License
 * and the GNU General Public License along with this program. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * The authors can be reached via e-mail to <support@deliantra.net>
 */

#include <global.h>
#include <sproto.h>
#include <skills.h>

static int
move_internal (object *op, char *params, int dir)
{
  if (params)
    {
      if (params[0] == 'f')
        {
          if (!op->contr->fire_on)
            {
              op->contr->fire_on = 1;
              move_player (op, dir);
              op->contr->fire_on = 0;
              return 0;
            }
        }
      else if (params[0] == 'r' && !op->contr->run_on)
        op->contr->run_on = 1;
    }

  move_player (op, dir);
  return 0;
}

int
command_east (object *op, char *params)
{
  return move_internal (op, params, 3);
}

int
command_north (object *op, char *params)
{
  return move_internal (op, params, 1);
}

int
command_northeast (object *op, char *params)
{
  return move_internal (op, params, 2);
}

int
command_northwest (object *op, char *params)
{
  return move_internal (op, params, 8);
}

int
command_south (object *op, char *params)
{
  return move_internal (op, params, 5);
}

int
command_southeast (object *op, char *params)
{
  return move_internal (op, params, 4);
}

int
command_southwest (object *op, char *params)
{
  return move_internal (op, params, 6);
}

int
command_west (object *op, char *params)
{
  return move_internal (op, params, 7);
}

int
command_stay (object *op, char *params)
{
  if (!op->contr->fire_on && (!params || params[0] != 'f'))
    return 0;

  fire (op, 0);
  return 0;
}

int
command_run (object *op, char *params)
{
  int dir = params ? atoi (params) : 0;

  if (dir < 0 || dir >= 9)
    {
      new_draw_info (NDI_UNIQUE, 0, op, "Can't run into a non adjacent square.");
      return 0;
    }

  op->direction = dir;
  op->contr->run_on = 1;

  return 1;
}

int
command_run_stop (object *op, char *params)
{
  op->contr->run_on = 0;
  return 1;
}

int
command_fire (object *op, char *params)
{
  int dir = params ? atoi (params) : 0;

  if (dir < 0 || dir >= 9)
    {
      new_draw_info (NDI_UNIQUE, 0, op, "Can't fire to a non adjacent square.");
      return 0;
    };

  op->direction = dir;
  op->contr->fire_on = 1;

  return 1;
}

int
command_fire_stop (object *op, char *params)
{
  op->contr->fire_on = 0;
  return 1;
}

